from ._daily_resource import (DailyResourceLocation,
                              DailyAccumulationResourceAsset,
                              DailyResourceContours)


class DailySolarRadiation(object):

    @classmethod
    def location(cls, lat, lon, start=None, end=None, unit=None, **kwargs):
        return _DailySolarByLocation.find(latitude=lat, longitude=lon,
                                          start=start, end=end, unit=unit, **kwargs)

    @classmethod
    def asset(cls, asset_uuid, start=None, end=None, unit=None, **kwargs):
        return _DailySolarByAsset.find(asset_uuid=asset_uuid, start=start, end=end,
                                       unit=unit, **kwargs)

    @classmethod
    def contours(cls, asset_uuid, start=None, end=None, unit=None, **kwargs):
        return _DailySolarContours.find(asset_uuid=asset_uuid, start=start, end=end,
                                        unit=unit, **kwargs)


class _DailySolarByLocation(DailyResourceLocation):

    _path = '/daily-solar-radiation/{latitude}/{longitude}'

    _deserialize = DailyResourceLocation._deserialize.extend({
        'solarRadiation': float
    })

    _serialize = DailyResourceLocation._serialize.extend({
        'solarRadiation': float
    })


class _DailySolarByAsset(DailyAccumulationResourceAsset):

    _path = '/daily-solar-radiation/{asset_uuid}'

    _deserialize = DailyAccumulationResourceAsset._deserialize.extend({
        'solarRadiation': float
    })

    _serialize = DailyAccumulationResourceAsset._serialize.extend({
        'solarRadiation': float
    })


class _DailySolarContours(DailyResourceContours):

    _path = '/daily-solar-radiation/{asset_uuid}/contours'



from voluptuous import Any

from ._hourly_resource import (HourlyResourceLocation,
                               HourlyTimeSeriesResourceAsset,
                               HourlyResourceContourByValidTime)


class HourlyTemperature(object):

    @classmethod
    def location(cls, lat, lon, start=None, end=None, unit=None, **kwargs):
        return _HourlyTempByLocation.find(latitude=lat, longitude=lon,
                                          start=start, end=end, unit=unit, **kwargs)

    @classmethod
    def asset(cls, asset_uuid, start=None, end=None, unit=None, **kwargs):
        return _HourlyTempByAsset.find(asset_uuid=asset_uuid, start=start, end=end,
                                       unit=unit, **kwargs)

    @classmethod
    def contours(cls, asset_uuid, valid_time=None, unit=None, **kwargs):
        return _HourlyTempContours.find(asset_uuid=asset_uuid, validTime=valid_time,
                                        unit=unit, **kwargs)


class _HourlyTempByLocation(HourlyResourceLocation):

    _path = '/hourly-temperature/{latitude}/{longitude}'

    _args = HourlyResourceLocation._args.extend({
        'unit': Any('Celsius', 'Fahrenheit', 'celsius', 'fahrenheit')
    })


class _HourlyTempByAsset(HourlyTimeSeriesResourceAsset):

    _path = '/hourly-temperature/{asset_uuid}'

    _args = HourlyTimeSeriesResourceAsset._args.extend({
        'unit': Any('Celsius', 'Fahrenheit', 'celsius', 'fahrenheit')
    })


class _HourlyTempContours(HourlyResourceContourByValidTime):

    _path = '/hourly-temperature/{asset_uuid}/contours'

    _args = HourlyResourceContourByValidTime._args.extend({
        'unit': Any('Celsius', 'Fahrenheit', 'celsius', 'fahrenheit')
    })



from voluptuous import Any, Optional, Schema
from skywiserestclient.validation import date, date_to_str, latitude, longitude, polygon, multipolygon

from skywiseinsight import InsightResource


class DailyResource(InsightResource):

    _args = Schema({
        'start': date_to_str,
        'end': date_to_str
    })

    _deserialize = Schema({
        'startDate': date,
        'endDate': date,
        'unit': {
            'description': unicode,
            'label': unicode
        }
    }, required=True)

    _serialize = Schema({
        'startDate': date_to_str,
        'endDate': date_to_str,
        'unit': {
            'description': unicode,
            'label': unicode
        }
    })


class DailyResourceLocation(DailyResource):

    _deserialize = DailyResource._deserialize.extend({
        'latitude': latitude,
        'longitude': longitude,
        'series': [{
            'value': float,
            'validDate': date
        }]
    })

    _serialize = DailyResource._serialize.extend({
        'latitude': latitude,
        'longitude': longitude,
        'series': [{
            'value': float,
            'validDate': date_to_str
        }]
    })


class DailyResourceAsset(DailyResource):

    _deserialize = DailyResource._deserialize.extend({
        'asset': unicode,
        Optional('contours'): unicode,
        'series': [{
            'validDate': date,
            'value': float,
            'contours': unicode,
            'assetStatistics': {
                'mean': float,
                'minimum': float,
                'maximum': float
            }
        }]
    })

    _serialize = DailyResource._serialize.extend({
        'asset': unicode,
        'contours': unicode,
        'series': [{
            'validDate': date_to_str,
            'value': float,
            'contours': unicode,
            'assetStatistics': {
                'mean': float,
                'minimum': float,
                'maximum': float
            }
        }]
    })


class DailyTimeSeriesResourceAsset(DailyResourceAsset):

    _deserialize = DailyResourceAsset._deserialize.extend({
        'timeSeriesStatistics': {
            'mean': float,
            'minimum': float,
            'maximum': float
        }
    })

    _serialize = DailyResourceAsset._serialize.extend({
        'timeSeriesStatistics': {
            'mean': float,
            'minimum': float,
            'maximum': float
        }
    })


class DailyAccumulationResourceAsset(DailyResourceAsset):

    _deserialize = DailyResourceAsset._deserialize.extend({
        'accumulationStatistics': {
            'mean': float,
            'minimum': float,
            'maximum': float
        }
    })

    _serialize = DailyResourceAsset._serialize.extend({
        'accumulationStatistics': {
            'mean': float,
            'minimum': float,
            'maximum': float
        }
    })


class DailyResourceContours(DailyResource):

    _deserialize = DailyResource._deserialize.extend({
        'asset': unicode,
        'type': unicode,
        'features': [{
            'type': unicode,
            'properties': {
                unicode: float
            },
            'geometry': Any(polygon, multipolygon)
        }]
    })

    _serialize = DailyResource._serialize.extend({
        'asset': unicode,
        'type': unicode,
        'features': [{
            'type': unicode,
            'properties': {
                unicode: float
            },
            'geometry': Any(polygon, multipolygon)
        }]
    })


class DailyResourceContourByValidDate(InsightResource):

    _args = Schema({
        'validDate': date_to_str
    })

    _deserialize = Schema({
        'validDate': date,
        'asset': unicode,
        'type': unicode,
        'features': [{
            'type': unicode,
            'properties': {
                unicode: float
            },
            'geometry': Any(polygon, multipolygon)
        }]
    })

    _serialize = Schema({
        'validDate': date_to_str,
        'asset': unicode,
        'type': unicode,
        'features': [{
            'type': unicode,
            'properties': {
                unicode: float
            },
            'geometry': Any(polygon, multipolygon)
        }]
    })



from skywiserestclient.validation import datetime, datetime_to_str

from ._hourly_resource import (HourlyResourceLocation,
                               HourlyResource)


class HourlyWindDirection(object):

    @classmethod
    def location(cls, lat, lon, start=None, end=None, unit=None, **kwargs):
        return _HourlyWindDirectionByLocation.find(latitude=lat, longitude=lon,
                                                   start=start, end=end, unit=unit, **kwargs)

    @classmethod
    def asset(cls, asset_uuid, start=None, end=None, unit=None, **kwargs):
        return _HourlyWindDirectionByAsset.find(asset_uuid=asset_uuid, start=start, end=end,
                                                unit=unit, **kwargs)


class _HourlyWindDirectionByLocation(HourlyResourceLocation):

    _path = '/hourly-wind-direction/{latitude}/{longitude}'


class _HourlyWindDirectionByAsset(HourlyResource):

    _path = '/hourly-wind-direction/{asset_uuid}'

    _deserialize = HourlyResource._deserialize.extend({
        'asset': unicode,
        'series': [{
            'validTime': datetime,
            'value': float
        }]
    })

    _serialize = HourlyResource._serialize.extend({
        'asset': unicode,
        'series': [{
            'validTime': datetime_to_str,
            'value': float
        }]
    })



from voluptuous import Any
from skywiserestclient.validation import temp_to_str

from ._daily_resource import (DailyResourceLocation, DailyAccumulationResourceAsset,
                              DailyResourceContours)


class DegreeDaysByLocation(DailyResourceLocation):

    _deserialize = DailyResourceLocation._deserialize.extend({
        'baseTemperature': float,
        'degreeDays': float
    })

    _serialize = DailyResourceLocation._serialize.extend({
        'baseTemperature': float,
        'degreeDays': float
    })

    _args = DailyResourceLocation._args.extend({
        'base': temp_to_str,
        'unit': Any('Celsius', 'Fahrenheit', 'celsius', 'fahrenheit')
    })


class DegreeDaysByAsset(DailyAccumulationResourceAsset):

    _deserialize = DailyAccumulationResourceAsset._deserialize.extend({
        'baseTemperature': float,
        'degreeDays': float
    })

    _serialize = DailyAccumulationResourceAsset._serialize.extend({
        'baseTemperature': float,
        'degreeDays': float
    })

    _args = DailyAccumulationResourceAsset._args.extend({
        'base': temp_to_str,
        'unit': Any('Celsius', 'Fahrenheit', 'celsius', 'fahrenheit')
    })


class DegreeDaysAssetContours(DailyResourceContours):

    _deserialize = DailyResourceContours._deserialize.extend({
        'baseTemperature': float
    })

    _serialize = DailyResourceContours._serialize.extend({
        'baseTemperature': float
    })

    _args = DailyResourceContours._args.extend({
        'base': temp_to_str,
        'unit': Any('Celsius', 'Fahrenheit', 'celsius', 'fahrenheit')
    })



from voluptuous import Any

from ._hourly_resource import (HourlyResourceLocation,
                               HourlyTimeSeriesResourceAsset,
                               HourlyResourceContourByValidTime)


class HourlyEtTallCrop(object):

    @classmethod
    def location(cls, lat, lon, start=None, end=None, unit=None, **kwargs):
        return _HourlyEtTallCropByLocation.find(latitude=lat, longitude=lon,
                                                start=start, end=end, unit=unit, **kwargs)

    @classmethod
    def asset(cls, asset_uuid, start=None, end=None, unit=None, **kwargs):
        return _HourlyEtTallCropByAsset.find(asset_uuid=asset_uuid, start=start, end=end,
                                             unit=unit, **kwargs)

    @classmethod
    def contours(cls, asset_uuid, valid_time=None, unit=None, **kwargs):
        return _HourlyEtTallCropContours.find(asset_uuid=asset_uuid, validTime=valid_time,
                                              unit=unit, **kwargs)


class _HourlyEtTallCropByLocation(HourlyResourceLocation):

    _path = '/hourly-evapotranspiration-tall-crop/{latitude}/{longitude}'

    _args = HourlyResourceLocation._args.extend({
        'unit': Any('inches', 'millimeters')
    })


class _HourlyEtTallCropByAsset(HourlyTimeSeriesResourceAsset):

    _path = '/hourly-evapotranspiration-tall-crop/{asset_uuid}'

    _args = HourlyTimeSeriesResourceAsset._args.extend({
        'unit': Any('inches', 'millimeters')
    })


class _HourlyEtTallCropContours(HourlyResourceContourByValidTime):

    _path = '/hourly-evapotranspiration-tall-crop/{asset_uuid}/contours'

    _args = HourlyResourceContourByValidTime._args.extend({
        'unit': Any('inches', 'millimeters')
    })



from voluptuous import Any

from ._daily_resource import (DailyResourceLocation,
                              DailyTimeSeriesResourceAsset,
                              DailyResourceContours)


class DailyLowTemperature(object):

    @classmethod
    def location(cls, lat, lon, start=None, end=None, unit=None, **kwargs):
        return _DailyLowTemperatureByLocation.find(latitude=lat, longitude=lon,
                                                   start=start, end=end, unit=unit, **kwargs)

    @classmethod
    def asset(cls, asset_uuid, start=None, end=None, unit=None, **kwargs):
        return _DailyLowTemperatureByAsset.find(asset_uuid=asset_uuid, start=start, end=end,
                                                unit=unit, **kwargs)

    @classmethod
    def contours(cls, asset_uuid, start=None, end=None, unit=None, **kwargs):
        return _DailyLowTemperatureContours.find(asset_uuid=asset_uuid, start=start, end=end,
                                                 unit=unit, **kwargs)


class _DailyLowTemperatureByLocation(DailyResourceLocation):

    _path = '/daily-low-temperature/{latitude}/{longitude}'

    _args = DailyResourceLocation._args.extend({
        'unit': Any('celsius', 'fahrenheit')
    })


class _DailyLowTemperatureByAsset(DailyTimeSeriesResourceAsset):

    _path = '/daily-low-temperature/{asset_uuid}'

    _args = DailyTimeSeriesResourceAsset._args.extend({
        'unit': Any('celsius', 'fahrenheit')
    })


class _DailyLowTemperatureContours(DailyResourceContours):

    _path = '/daily-low-temperature/{asset_uuid}/contours'



from ._degree_days_resource import DegreeDaysByLocation, DegreeDaysByAsset, DegreeDaysAssetContours


class Hdd(object):

    @classmethod
    def location(cls, lat, lon, start=None, end=None, **kwargs):
        return _HddByLocation.find(latitude=lat, longitude=lon, start=start, end=end, **kwargs)

    @classmethod
    def asset(cls, asset_uuid, start=None, end=None, **kwargs):
        return _HddByAsset.find(asset_uuid, start=start, end=end, **kwargs)

    @classmethod
    def contours(self, asset_uuid, start=None, end=None, **kwargs):
        return _HddAssetContours.find(asset_uuid=asset_uuid, start=start, end=end, **kwargs)


class _HddByLocation(DegreeDaysByLocation):

    _path = "/heating-degree-days/{latitude}/{longitude}"


class _HddByAsset(DegreeDaysByAsset):

    _path = "/heating-degree-days"


class _HddAssetContours(DegreeDaysAssetContours):

    _path = "/heating-degree-days/{asset_uuid}/contours"



from voluptuous import Any

from ._hourly_resource import (HourlyResourceLocation,
                               HourlyAccumulationResourceAsset,
                               HourlyResourceContours)


class HourlyPrecipitation(object):

    @classmethod
    def location(cls, lat, lon, start=None, end=None, unit=None, **kwargs):
        return _HourlyPrecipByLocation.find(latitude=lat, longitude=lon,
                                            start=start, end=end, unit=unit, **kwargs)

    @classmethod
    def asset(cls, asset_uuid, start=None, end=None, unit=None, **kwargs):
        return _HourlyPrecipByAsset.find(asset_uuid=asset_uuid, start=start, end=end,
                                         unit=unit, **kwargs)

    @classmethod
    def contours(cls, asset_uuid, start=None, end=None, unit=None, **kwargs):
        return _HourlyPrecipContours.find(asset_uuid=asset_uuid, start=start, end=end,
                                          unit=unit, **kwargs)


class _HourlyPrecipByLocation(HourlyResourceLocation):

    _path = '/hourly-precipitation/{latitude}/{longitude}'

    _deserialize = HourlyResourceLocation._deserialize.extend({
        'precipitation': float
    })

    _serialize = HourlyResourceLocation._serialize.extend({
        'precipitation': float
    })

    _args = HourlyResourceLocation._args.extend({
        'unit': Any('inches', 'millimeters')
    })


class _HourlyPrecipByAsset(HourlyAccumulationResourceAsset):

    _path = '/hourly-precipitation/{asset_uuid}'

    _deserialize = HourlyAccumulationResourceAsset._deserialize.extend({
        'precipitation': float
    })

    _serialize = HourlyAccumulationResourceAsset._serialize.extend({
        'precipitation': float
    })

    _args = HourlyAccumulationResourceAsset._args.extend({
        'unit': Any('inches', 'millimeters')
    })


class _HourlyPrecipContours(HourlyResourceContours):

    _path = '/hourly-precipitation/{asset_uuid}/contours'



from voluptuous import Any

from ._daily_resource import (DailyResourceLocation,
                              DailyTimeSeriesResourceAsset,
                              DailyResourceContourByValidDate)


class DailyEtShortCrop(object):

    @classmethod
    def location(cls, lat, lon, start=None, end=None, unit=None, **kwargs):
        return _DailyEtShortCropByLocation.find(latitude=lat, longitude=lon,
                                                start=start, end=end, unit=unit,
                                                **kwargs)

    @classmethod
    def asset(cls, asset_uuid, start=None, end=None, unit=None, **kwargs):
        return _DailyEtShortCropByAsset.find(asset_uuid=asset_uuid, start=start,
                                             end=end, unit=unit, **kwargs)

    @classmethod
    def contours(cls, asset_uuid, validDate=None, unit=None, **kwargs):
        return _DailyEtShortCropContours.find(asset_uuid=asset_uuid, validDate=validDate, unit=unit, **kwargs)


class _DailyEtShortCropByLocation(DailyResourceLocation):

    _path = '/daily-evapotranspiration-short-crop/{latitude}/{longitude}'

    _args = DailyResourceLocation._args.extend({
        'unit': Any('inches', 'millimeters')
    })


class _DailyEtShortCropByAsset(DailyTimeSeriesResourceAsset):

    _path = '/daily-evapotranspiration-short-crop/{asset_uuid}'

    _args = DailyTimeSeriesResourceAsset._args.extend({
        'unit': Any('inches', 'millimeters')
    })


class _DailyEtShortCropContours(DailyResourceContourByValidDate):

    _path = '/daily-evapotranspiration-short-crop/{asset_uuid}/contours'

    _args = DailyResourceContourByValidDate._args.extend({
        'unit': Any('millimeters', 'inches')
    })



from voluptuous import Any, Schema
from skywiserestclient.validation import latitude, longitude, multipolygon, polygon

from skywiseinsight import InsightResource


class UnsupportedGeometryException(Exception):
    pass


class Asset(InsightResource):

    _path = "/assets"

    _deserialize = Schema({
        'id': unicode,
        'description': unicode,
        'type': unicode,
        'geometries': [Any(polygon, multipolygon)],
        'centroid': {
            'latitude': latitude,
            'longitude': longitude
        }
    })

    _serialize = Schema({
        'description': basestring,
        'type': basestring,
        'geometries': [Any(polygon, multipolygon)]
    })

    def __init__(self):
        super(Asset, self).__init__()
        self.type = 'GeometryCollection'
        self.geometries = []

    def add_geometry(self, g):
        if g.type == 'MultiPolygon':
            self._add_mp(g)
        elif g.type == 'Polygon':
            self._add_p(g)
        else:
            raise UnsupportedGeometryException('Only Polygons/MultiPolygons supported.')

    def _add_p(self, p):
        g = {
            'type': 'Polygon',
            'coordinates': p.coordinates
        }
        self.geometries.append(g)

    def _add_mp(self, mp):
        g = {
            'type': 'MultiPolygon',
            'coordinates': [mp.coordinates]
        }
        self.geometries.append(g)



from ._hourly_resource import (HourlyResourceLocation,
                               HourlyTimeSeriesResourceAsset,
                               HourlyResourceContourByValidTime)


class HourlyRelativeHumidity(object):

    @classmethod
    def location(cls, lat, lon, start=None, end=None, unit=None, **kwargs):
        return _HourlyRHByLocation.find(latitude=lat, longitude=lon,
                                        start=start, end=end, unit=unit, **kwargs)

    @classmethod
    def asset(cls, asset_uuid, start=None, end=None, unit=None, **kwargs):
        return _HourlyRHByAsset.find(asset_uuid=asset_uuid, start=start, end=end,
                                     unit=unit, **kwargs)

    @classmethod
    def contours(cls, asset_uuid, start=None, end=None, unit=None, **kwargs):
        return _HourlyRHContours.find(asset_uuid=asset_uuid, validTime=end,
                                      unit=unit, **kwargs)


class _HourlyRHByLocation(HourlyResourceLocation):

    _path = '/hourly-relative-humidity/{latitude}/{longitude}'


class _HourlyRHByAsset(HourlyTimeSeriesResourceAsset):

    _path = '/hourly-relative-humidity/{asset_uuid}'


class _HourlyRHContours(HourlyResourceContourByValidTime):

    _path = '/hourly-relative-humidity/{asset_uuid}/contours'
